SystemOrganization addCategory: #'Seaside-Tests-Flow-Functional'!
SystemOrganization addCategory: #'Seaside-Tests-Flow-Unit'!

WAFunctionalTest subclass: #WACallDuringRenderFunctionalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WACallDuringRenderFunctionalTest methodsFor: 'rendering' stamp: 'jf 10/28/2008 10:15'!
renderContentOn: html
	html paragraph: [
		html anchor
			callback: [ self call: WACallDuringRenderTestComponent new ];
			with: 'This link'.
		html text: ' will call a Component which attempts to do a #call: in its #renderContentOn: method. This is not allowed and should trigger a descriptive error. With WAWalkbackErrorHandler enabled, the output should explain to the user the likely cause of the problem.' ]! !

WAFunctionalTest subclass: #WADecorationTest
	instanceVariableNames: 'task'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WADecorationTest commentStamp: 'lr 5/31/2008 10:58' prior: 0!
http://code.google.com/p/seaside/issues/detail?id=71!

!WADecorationTest methodsFor: 'accessing' stamp: 'lr 5/31/2008 11:18'!
children
	^ Array with: task! !

!WADecorationTest methodsFor: 'initialization' stamp: 'lr 5/31/2008 15:50'!
initialize
	super initialize.
	task := WADecorationTask new! !

!WADecorationTest methodsFor: 'rendering' stamp: 'jf 10/22/2008 16:13'!
renderContentOn: html
	html orderedList: [
		html listItem: 'Click button of Step 1'.
		html listItem: 'Click back-button'.
		html listItem: 'Click button of Step 1'.
		html listItem: 'Click button of Step 2' ].
	html paragraph: 'Step 1 and Step 2 should have consistent number of visible presenters. Especially there should never be two WAAnswerDecorations.'.
	
	html heading level: 2; with: 'Application'.
	html render: task.
	
	html heading level: 2; with: 'Visible Presenters'.
	html table: [
		html tableRow: [
			html tableHeading: 'Presenter'.
			html tableHeading.
			html tableHeading: 'Next'.
			html tableHeading ].
		self visiblePresentersDo: [ :each |
			html tableRow: [
				html tableData: each printString.
				html tableData: each identityHash.
				html tableData: (each isDecoration
					ifTrue: [ each next printString ]
					ifFalse: [ each decoration printString ]).
				html tableData: (each isDecoration
					ifTrue: [ each next identityHash ]
					ifFalse: [ each decoration identityHash ]) ] ] ]! !

WAFunctionalTest subclass: #WAFilterFunctionalTest
	instanceVariableNames: 'child'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAFilterFunctionalTest methodsFor: 'actions' stamp: 'lr 8/31/2008 15:17'!
authentication
	self authenticateWith: self during: [ child inform: 'Congratulations, you have been authenticated. The new filter should be visible in the list above. Clicking on "ok" should remove the filter. Using the back button and hitting refresh should show the filter again.' ]! !

!WAFilterFunctionalTest methodsFor: 'accessing' stamp: 'lr 8/31/2008 15:12'!
children
	^ Array with: child! !

!WAFilterFunctionalTest methodsFor: 'initialization' stamp: 'lr 8/31/2008 15:12'!
initialize
	super initialize.
	child := WAComponent new! !

!WAFilterFunctionalTest methodsFor: 'actions' stamp: 'lr 8/31/2008 15:15'!
protector
	self filterWith: WAProtectionFilter new during: [ child inform: 'Open this page on a different computer by copying the complete URL to a machine with a different IP address. Seaside should reply with a forbidden message. Closing this dialog should remove the filter.' ]! !

!WAFilterFunctionalTest methodsFor: 'rendering' stamp: 'lr 8/31/2008 15:16'!
renderActionsOn: html
	html paragraph: [
		html anchor
			callback: [ self authentication ];
			with: 'Basic Authentication'.
		html text: ' (admin/seaside)'.
		html break.
		html anchor
			callback: [ self protector ];
			with: 'Session Protector' ].
	html horizontalRule.
	html paragraph: self children! !

!WAFilterFunctionalTest methodsFor: 'rendering' stamp: 'lr 8/31/2008 15:08'!
renderContentOn: html
	self renderFiltersOn: html.
	self renderActionsOn: html! !

!WAFilterFunctionalTest methodsFor: 'rendering' stamp: 'lr 11/4/2008 09:22'!
renderFiltersOn: html
	html paragraph: 'Below a list of filters wrapping the current session:'.
	html orderedList list: self session filters! !

!WAFilterFunctionalTest methodsFor: 'private' stamp: 'lr 8/31/2008 15:01'!
verifyPassword: aPasswordString forUser: aUserString
	^ aUserString = 'admin' and: [ aPasswordString = 'seaside' ]! !

WAFunctionalTest subclass: #WAFlowFunctionalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAFlowFunctionalTest methodsFor: 'actions' stamp: 'pmm 8/31/2008 14:32'!
goAnchors
	| component |
	1 to: 5 do: [ :each | 
		component := WAComponent new.
		component addMessage: [ :html | 
			html anchor
				callback: [ component answer ];
				with: each seasideString , ': ' , self stackDepth seasideString ] fixCallbackTemps.
		self call: component ]! !

!WAFlowFunctionalTest methodsFor: 'actions' stamp: 'pmm 8/31/2008 14:33'!
goButtons
	1 to: 5 do: [ :each |
		self inform: each seasideString, ': ', self stackDepth seasideString ]! !

!WAFlowFunctionalTest methodsFor: 'rendering' stamp: 'lr 5/22/2007 18:10'!
renderContentOn: html
	html paragraph: 'The following two anchors should trigger flows with 5 steps each. The stack should neither grow nor shrink. Backtracking and spawning of windows should properly work.'.
	html paragraph: [
		html anchor
			callback: [ self goAnchors ];
			with: 'go anchors'.
		html break.
		html anchor
			callback: [ self goButtons ];
			with: 'go buttons' ]! !

!WAFlowFunctionalTest methodsFor: 'private' stamp: 'jf 1/22/2009 02:43'!
stackDepth

	^WAPlatform current stackDepth! !

WAFunctionalTest subclass: #WAManyTasksFunctionalTest
	instanceVariableNames: 'tasks selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAManyTasksFunctionalTest methodsFor: 'actions' stamp: 'jf 2/5/2009 08:54'!
addTask
	selection contents notNil ifTrue: [ tasks add: selection contents new ]! !

!WAManyTasksFunctionalTest methodsFor: 'actions' stamp: 'jf 2/5/2009 08:54'!
callTask
	selection contents notNil ifTrue: [ self call: selection contents new ]! !

!WAManyTasksFunctionalTest methodsFor: 'hooks' stamp: 'jf 2/5/2009 08:43'!
children
	^ tasks! !

!WAManyTasksFunctionalTest methodsFor: 'initialization' stamp: 'jf 2/8/2009 11:33'!
initialize
	super initialize.
	tasks := OrderedCollection new.
	selection := WAValueHolder new.
	tasks add: WAConvenienceFunctionalTest new! !

!WAManyTasksFunctionalTest methodsFor: 'rendering' stamp: 'jf 2/5/2009 08:54'!
renderContentOn: html
	html form: [ html div: [
		html select
			list: (WATask allSubclasses asSortedCollection: [ :a :b | a name <= b name ]);
			on: #contents of: selection.
		html submitButton
			callback: [ self addTask ];
			with: 'Add'.
		html submitButton
			callback: [ self callTask ];
			with: 'Call' ]].
	self renderTasksOn: html! !

!WAManyTasksFunctionalTest methodsFor: 'rendering' stamp: 'jf 2/8/2009 11:36'!
renderTasksOn: html
	tasks do: [ :each |
		html div: [
			html div: [ html render: each ].
			html anchor
				callback: [ tasks remove: each ];
				with: '(remove task)' ] ]! !

!WAManyTasksFunctionalTest methodsFor: 'initialization' stamp: 'jf 2/8/2009 10:50'!
states
	^ Array with: tasks! !

WAFunctionalTest subclass: #WATransactionFunctionalTest
	instanceVariableNames: 'nestedTransaction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WATransactionFunctionalTest commentStamp: 'pmm 9/2/2007 00:08' prior: 0!
A WATransactionTest runs a WANestedTransaction with a description!

!WATransactionFunctionalTest methodsFor: 'accessing' stamp: 'lr 2/8/2009 15:54'!
children
	^ Array with: nestedTransaction! !

!WATransactionFunctionalTest methodsFor: 'initialization' stamp: 'lr 2/8/2009 15:54'!
initialize
	super initialize.
	nestedTransaction := WANestedTransaction new! !

!WATransactionFunctionalTest methodsFor: 'rendering' stamp: 'lr 2/8/2009 15:54'!
renderContentOn: html 
	self renderExplanationOn: html.
	html render: nestedTransaction! !

!WATransactionFunctionalTest methodsFor: 'rendering' stamp: 'pmm 9/13/2008 14:23'!
renderExplanationOn: html
	html paragraph: 'This checks if nested #isolate: block work. It has the following nested transactions:'.
	html orderedList: [
		html listItem: 'Inside parent txn'.
		html listItem: [
			html orderedList: [
				html listItem: 'Inside child txn' ] ].
		html listItem: 'Outside child txn' ].
	html paragraph: 'If you leave the child transaction and enter it with the back button you should end up in the parent transaction. If you leave the parent transaction with the back button and enter either it or the child transaction with the back button you should end up outside the parent transaction.'! !

!WAPackage class methodsFor: '*seaside-tests-flow' stamp: 'jf 1/27/2009 11:34'!
seasideTestsFlow
	^ self new
		name: 'Seaside-Tests-Flow';
		description: 'Functional tests for Seaside-Flow.';
		addDependency: 'Seaside-Flow';
		addDependency: 'Seaside-Tests-Functional';
		yourself! !

WATask subclass: #WADecorationTask
	instanceVariableNames: 'component1 component2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WADecorationTask methodsFor: 'accessing' stamp: 'lr 11/28/2008 16:40'!
children
	"Even if we component1 and component2, we have to return them as children as otherwise their decorations are not properly backtracked. See WADecorationTest."
	
	^ Array with: component1 with: component2! !

!WADecorationTask methodsFor: 'running' stamp: 'lr 5/31/2008 10:36'!
go
	self call: component1.
	self call: component2! !

!WADecorationTask methodsFor: 'initialization' stamp: 'lr 5/31/2008 15:53'!
initialize
	super initialize.
	component1 := WAFormDialog new
		addMessage: 'Step 1';
		yourself.
	component2 := WAFormDialog new
		addMessage: 'Step 2';
		yourself! !

WATask subclass: #WAEmptyTestTask
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAEmptyTestTask methodsFor: 'running' stamp: 'jf 2/8/2009 15:09'!
go
	"do nothing. I wonder what happens?"! !

WATask subclass: #WAFunctionalTaskTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

WAFunctionalTaskTest subclass: #WAConvenienceFunctionalTest
	instanceVariableNames: 'cheese'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'lr 10/28/2007 14:42'!
chooseCheese
	cheese := self
		chooseFrom: #('Greyerzer' 'Tilsiter' 'Sbrinz')
		caption: 'What''s your favorite Cheese?'.
	cheese isNil ifTrue: [ self chooseCheese ]! !

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'lr 10/28/2007 14:42'!
confirmCheese
	^self confirm: 'Is ', cheese,  ' your favorite cheese?'! !

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'pmm 6/30/2007 12:24'!
go
	[ self chooseCheese.
	  self confirmCheese ] whileFalse.
	self informCheese! !

!WAConvenienceFunctionalTest methodsFor: 'controlling' stamp: 'lr 10/28/2007 14:42'!
informCheese
	self inform: 'Your favorite cheese is ', cheese, '.'! !

!WAConvenienceFunctionalTest methodsFor: 'accessing' stamp: 'pmm 8/29/2008 16:05'!
states
	^Array with: self! !

WAFunctionalTaskTest subclass: #WAExceptionFunctionalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WAExceptionFunctionalTest methodsFor: 'processing' stamp: 'pmm 11/1/2007 21:32'!
go
	[ (self confirm: 'Raise an exception?')
		ifTrue: [ self error: 'foo' ] ]
			on: Error
			do: [ :error | self inform: 'Caught: ', error description ]! !

!WAFunctionalTaskTest class methodsFor: 'testing' stamp: 'jf 2/1/2009 21:29'!
isFunctionalTest
	^ (self name = #WAFunctionalTaskTest) not! !

WATask subclass: #WANestedTransaction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WANestedTransaction commentStamp: 'pmm 9/2/2007 00:08' prior: 0!
A WANestedTransaction is a test that uses two nested #isolate: blocks!

!WANestedTransaction methodsFor: 'processing' stamp: 'lr 10/28/2007 14:42'!
go
	self inform: 'Before parent txn'.
	self isolate:
			[self inform: 'Inside parent txn'.
			self isolate: [self inform: 'Inside child txn'].
			self inform: 'Outside child txn'].
	self inform: 'Outside parent txn'! !

WAComponent subclass: #WACallDuringRenderTestComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Functional'!

!WACallDuringRenderTestComponent methodsFor: 'rendering' stamp: 'jf 10/28/2008 10:13'!
renderContentOn: html
	self call: WAMiniCalendar new! !

TestCase subclass: #WACurrentSessionContinuationTest
	instanceVariableNames: 'savedContinuation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Unit'!

!WACurrentSessionContinuationTest methodsFor: 'running' stamp: 'jf 2/3/2009 13:31'!
tearDown
	"GemStone can have trouble if a continuation holding a Semaphore is persisted"
	savedContinuation := nil.
	super tearDown! !

!WACurrentSessionContinuationTest methodsFor: 'testing' stamp: 'lr 2/8/2009 15:46'!
testBalancedContextContinuations
	"Cause the ensure blocks in WAContext to fire - use balanced nesting and things work fine"
	| x session alternateSession tertiarySession quaternarySession context |
	session := WASession new.
	alternateSession := WASession new.
	tertiarySession := WASession new.
	quaternarySession := WASession new.
	context := WARequestContext request: nil response: nil.
	WACurrentRequestContext 
		use: context
		during: 
			[ WACurrentRequestContext value 
				push: tertiarySession
				while: 
					[ self assert: WACurrentRequestContext value session == tertiarySession.
					WACurrentRequestContext value 
						push: session
						while: 
							[ x := WAContinuation currentDo: 
								[ :cc | 
								savedContinuation := cc.
								0 ].
							x == 0 
								ifTrue: [ self assert: WACurrentRequestContext value session == session ]
								ifFalse: 
									[ self assert: x = 1.
									self assert: WACurrentRequestContext value session == quaternarySession ] ].
					x == 0 ifTrue: 
						[ self assert: WACurrentRequestContext value session == tertiarySession ].
					x == 1 ifTrue: 
						[ self assert: WACurrentRequestContext value session == alternateSession ] ] ].
	x ~~ 1 ifTrue: 
		[ WACurrentRequestContext 
			use: context
			during: 
				[ WACurrentRequestContext value 
					push: alternateSession
					while: 
						[ WACurrentRequestContext value 
							push: quaternarySession
							while: [ savedContinuation value: 1 ] ] ] ]! !

!WACurrentSessionContinuationTest methodsFor: 'testing' stamp: 'jf 2/3/2009 13:31'!
testContinuationInheritance
	"Verify that continuation inherits it's context from parent process"
	| x session alternateSession context |
	session := WASession new.
	alternateSession := WASession new.
	context := WARequestContext request: nil response: nil.
	WACurrentRequestContext 
		use: context
		during: 
			[ WACurrentRequestContext value 
				push: session
				while: 
					[ x := WAContinuation currentDo: 
						[ :cc | 
						savedContinuation := cc.
						0 ].
					x == 0 
						ifTrue: [ self assert: WACurrentRequestContext value session == session ]
						ifFalse: 
							[ self assert: x = 1.
							self assert: WACurrentRequestContext value session == alternateSession ] ] ].
	x ~~ 1 ifTrue: 
		[ WACurrentRequestContext 
			use: context
			during: 
				[ WACurrentRequestContext value 
					push: alternateSession
					while: [ savedContinuation value: 1 ] ] ]! !

!WACurrentSessionContinuationTest methodsFor: 'testing' stamp: 'jf 2/3/2009 13:31'!
testContinuations
	"verify that process variable not copied by continuation."
	| x session context |
	session := WASession new.
	context := WARequestContext request: nil response: nil.
	WACurrentRequestContext 
		use: context
		during: 
			[ WACurrentRequestContext value 
				push: session
				while: 
					[ x := WAContinuation currentDo: 
						[ :cc | 
						savedContinuation := cc.
						0 ].
					x == 0 ifTrue: 
						[ self assert: WACurrentSession value == session.
						"invoke continuation for the first time - reuse process variables"
						savedContinuation value: 1 ].
					x == 1 ifTrue: 
						[ self assert: WACurrentSession value = session.
						WACurrentRequestContext reset.
						"invoke continuation for the second time - process variables reset"
						savedContinuation value: 2 ].
					"no value for WACurrentSession after continuation invoked for second time"
					self assert: x = 2.
					self assert: WACurrentSession value isNil ] ]! !

!WACurrentSessionContinuationTest methodsFor: 'testing' stamp: 'jf 2/3/2009 13:31'!
testSimpleContinuation
	"verify that continuation reuses the process variable from the current process."
	| x continuation session context |
	session := WASession new.
	context := WARequestContext request: nil response: nil.
	WACurrentRequestContext 
		use: context
		during: 
			[ WACurrentRequestContext value 
				push: session
				while: 
					[ x := WAContinuation currentDo: 
						[ :cc | 
						continuation := cc.
						false ].
					self assert: WACurrentSession value = session.
					x ifFalse: [ continuation value: true ].
					self assert: x ] ]! !

TestCase subclass: #WAFlowPlatformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Flow-Unit'!

!WAFlowPlatformTest methodsFor: 'private' stamp: 'jf 1/27/2009 14:21'!
evaluateCallbackContinuation: aContinuation with: anObject
	| callback |
	callback := WAActionCallback new.
	callback block: [ aContinuation value: anObject ].
	[ callback evaluateWithArgument: nil ]
		on: WARenderNotification do: [ :n | ]! !

!WAFlowPlatformTest methodsFor: 'testing' stamp: 'jf 1/27/2009 14:20'!
testSuspendCallbackDo
	| continuation result |
	continuation := nil.
	self withCallbackContinuationDo: [ :cc | continuation := cc ] resultBlock: [ :r | result := r ].
	self assert: continuation notNil.
	self evaluateCallbackContinuation: continuation with: 123.
	self assert: result = 123! !

!WAFlowPlatformTest methodsFor: 'private' stamp: 'jf 2/13/2009 15:54'!
withCallbackContinuationDo: ccBlock resultBlock: resultBlock
	| callback |
	callback := WAActionCallback new.
	callback block: [
		resultBlock value: (WAPlatform current suspendFlowDo: ccBlock) ].
	[ callback evaluateWithArgument: nil ]
		on: WARenderNotification do: [ :n | ].! !
